## This file is a part of PyGG: PyGTK and Glade
## (C) 2005 Franck Pommereau <pommereau@univ-paris12.fr>
##
## This library is free software; you can redistribute it and/or
## modify it under the terms of the GNU Lesser General Public
## License as published by the Free Software Foundation; either
## version 2.1 of the License, or (at your option) any later version.
##
## This library is distributed in the hope that it will be useful,
## but WITHOUT ANY WARRANTY; without even the implied warranty of
## MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
## Lesser General Public License for more details.
##
## You should have received a copy of the GNU Lesser General Public
## License along with this library; if not, write to the Free Software
## Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

import os.path, glob, zipfile, warnings
import gtk, gtk.glade

class GladeSources :
    """A class to hold references to the various Glade files that PyGG
    will be able to load. A source may be a Glade file, a ZIP file or
    a directory. In the two last cases, all the Glade files contained
    in the source are considered (a directory is visited only at its
    first level). Each considered Glade file is given a name, eg,
    'FileName.glade' will be named 'filename' (lower case)."""
    def __init__ (self, *sources) :
        "Create an instance with initial sources."
        self._source = {}
        for source in sources :
            self.add(source)
    def _src (self, path) :
        return os.path.basename(path).lower().rsplit(".", 1)[0]
    def _add_file (self, name) :
        src = self._src(name)
        if src in self._source :
            warnings.warn("duplicated source '%s', ignored" % name,
                          stacklevel=3)
        else :
            self._source[src] = name
    def _add_zipped (self, name, archive, archivename) :
        src = self._src(name)
        if src in self._source :
            warnings.warn("duplicated source '%s/%s', ignored"
                          % (archivename, name), stacklevel=3)
        else :
            self._source[src] = (archive, name)
    def add (self, source) :
        "Add a source."
        if os.path.islink(source) :
            source = os.path.realpath(source)
        if os.path.isdir(source) :
            for name in glob.glob(os.path.join(source, "*.glade")) :
                self._add_file(name)
        elif os.path.isfile(source) :
            if source.endswith(".glade") :
                self._add_file(source)
            else :
                try :
                    archive = zipfile.ZipFile(source)
                    for name in archive.namelist() :
                        if name.endswith(".glade") :
                            self._add_zipped(name, archive, source)
                except :
                    raise ValueError, "invalid path '%s'" % source
        else :
            raise ValueError, "invalid path '%s'" % source
    def __del__ (self) :
        for value in self._source.values() :
            try :
                value[0].close()
            except :
                pass
    def read (self, name) :
        "Return the content of a source designated through its name"
        key = name.lower()
        try :
            if isinstance(self._source[key], str)  :
                return open(self._source[key]).read()
            else :
                return self._source[key][0].read(self._source[key][1])
        except :
            raise RuntimeError, "invalid Glade source '%s'" % name

sources = GladeSources()

class Pygg :
    """The base class for Glade+PyGTK classes. A subclass named
    'MyPygg' will be linked to the Glade source named 'mypygg' and the
    widget named 'MyPygg' it defines. This widget will be accessible
    through the member variable 'MyPygg' in the instances of the
    class. The attributes of the widget are directly available in the
    instances of MyPygg, the other widgets defined in the Glade file
    are also available as attributes of the instances (the name of the
    widget is the name of the attribute). Callbacks defined in the
    Glade file are connected to the methods found in the subclass."""
    def __init__ (self) :
        self._cached_attrs = set()
        self._build()
    def __del__ (self) :
        try :
            getattr(self, self.__class__.__name__).destroy()
        except :
            pass
    def _build (self, *args) :
        global sources
        for name in self._cached_attrs :
            delattr(self, name)
        self._cached_attrs.clear()
        classname = self.__class__.__name__
        if not hasattr(self.__class__, "__glade__") :
            self.__class__.__glade__ = sources.read(classname)
        xml = gtk.glade.xml_new_from_buffer(self.__glade__,
                                            len(self.__glade__),
                                            root=classname)
        widget = xml.get_widget(classname)
        if widget is None :
            raise RuntimeError, "could not get widget '%s'" % classname
        callbacks = {}
        for name, fun in self.__class__.__dict__.items() :
            if callable(fun) :
                callbacks[name] = getattr(self, name)
        xml.signal_autoconnect(callbacks)
        # copying widgets
        for widget in xml.get_widget_prefix(''):
            name = widget.get_name()
            assert not hasattr(self, name), "cannot overwite attribute '%s'" % name
            setattr(self, name, widget)
            self._cached_attrs.add(name)
    def __getattr__ (self, name) :
        try :
            widget = getattr(self, self.__class__.__name__)
            attr = getattr(widget, name)
            setattr(self, name, attr)
            self._cached_attrs.add(name)
            return attr
        except AttributeError :
            raise AttributeError, ("'%s' object no attribute '%s'"
                                   % (self.__class__.__name__, name))

class Widget (Pygg) :
    """An arbitrary widget. In order to define it with Glade, put it
    inside a window which will be ignored when the PyGG object is
    created."""
    pass

class Window (Widget) :
    "A top level window, which is rebuilt when destroyed"
    def _build (self) :
        Pygg._build(self)
        def rebuild (*args) :
            self._build()
        self.connect("destroy-event", rebuild)

class AppWindow (Window) :
    """The main window of an application. It has methods run and quit
    to start/quit the GTK main loop."""
    def run (self) :
        self.show_all()
        gtk.main()
    def quit (self, *args) :
        gtk.main_quit()

class Dialog (Window) :
    """A dialog window. It has a method run, as Dialog widgets, but
    which also hode the dialog on return."""
    def _build (self, *args) :
        Window._build(self, *args)
    def run (self) :
        result = getattr(self, self.__class__.__name__).run()
        self.hide()
        return result
